/*
    mandelbmp - render the Mandelbrot set or Julia sets as BMP files
    Copyright (C) 2022 cas <cas@sdf.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "cplx.h"
#include <math.h>

/*
 * Arithmetic on complex numbers. A bit weird, tbh. 
 * */

void cplx_add(cplx* a, cplx* b){
  // adds b to a
  
  a -> real += b -> real;
  a -> imag += b -> imag;
}

void cplx_mul(cplx* a, cplx* b){
  // multiplies a to b and stores result in a
  double res_r = (a -> real) * (b -> real) - (a -> imag) * (b -> imag);
  double res_i = (a -> real) * (b -> imag) + (a -> imag) * (b -> real);
  
  a -> real = res_r;
  a -> imag = res_i;
}

void cplx_mul_real(cplx* a, double s){
  // multiplies real number s to a
  
  a -> real *= s;
  a -> imag *= s;
}

void cplx_set_polar(cplx *z, double r, double phi){
  z -> real = r * cos(phi);
  z -> imag = r * sin(phi);
}


void cplx_power(cplx* z, double exponent){
  double r   = sqrt(z -> real * z -> real + z -> imag * z -> imag);
  double phi = atan2(z -> imag, z -> real);
  
  cplx_set_polar(z, pow(r, exponent), phi * exponent);
}
